<?php

namespace App\HttpController\Google;

use App\Key\K;
use App\Tools\GSession;
use App\Tools\Proxy;
use App\Tools\ProxyManager;
use App\Tools\Shell;
use App\Tools\StreetViewDownloader;
use App\Tools\System;
use App\Tools\WechatNotice;
use EasySwoole\Component\TableManager;
use EasySwoole\Component\Timer;
use EasySwoole\EasySwoole\Config;
use EasySwoole\EasySwoole\ServerManager;
use Swlib\Saber;
use Swlib\SaberGM;
use Swoole\Coroutine;

class Handler
{
    public function workerStart($server, $worker_id)
    {
//        System::log($worker_id, 'worker start');
        $this->historyWorkerStart($worker_id);
        $config=Config::getInstance()->getConf('BIGEMAP_DATA');
//        System::config(K::KEY_ENABLE_STREET_VIEW_DOWNLOAD)
        if ($config['street_view']&&!file_exists(dirname(EASYSWOOLE_ROOT).'/manager/main_server')){
            $downloader=new StreetViewDownloader();
            $downloader->workerStart($worker_id);
        }
    }

    public function historyWorkerStart($worker_id){
        if ($worker_id !== 0) return;
        $this->curlSession();
        Timer::getInstance()->loop(5000, [$this, 'curlSession']);
        Timer::getInstance()->after(1000, function () {
            //如果当前的端口号是
            $port = System::config(K::KEY_SERVER_PORT);
            if ($port != 9502) return;
            $baseDir = dirname(EASYSWOOLE_ROOT);
            $dirs = scandir($baseDir);
            foreach ($dirs as $v) {
                if (strpos($v, 'easyswoole') === 0) {
                    $serverPort = str_replace('easyswoole', '', $v);
//                    System::log(Shell::checkProcess('bigemap' . $serverPort),'check process '.$serverPort,1);
                    if (is_numeric($serverPort) && $serverPort > 9502 && is_dir($baseDir . '/cache/' . $serverPort) && !Shell::checkProcess('bigemap' . $serverPort)) {
                        echo 'php ' . $baseDir . '/' . $v . '/easyswoole server start -d -mode=produce', PHP_EOL;
                        Shell::shellStart('php ' . $baseDir . '/' . $v . '/easyswoole server start -d -mode=produce');
                    }
                }
            }
        });
        Timer::getInstance()->loop(1000, [$this, 'everySecond']);
    }
    public function pipeMessage($server, $src_id, $message)
    {
        switch ($message['action']) {
            case 'allow':
                Coroutine::resume($message['cid']);
                break;
        }
    }

    public function sendData()
    {
        $cacheData = System::serverData();
        if (!$cacheData) return;
        $server = ServerManager::getInstance()->getSwooleServer();
        $table = TableManager::getInstance()->get(K::TABLE_WEB_FD);
        foreach ($table as $k => $v) {
            $info = json_decode($v['data'], true);
            if ($info['type'] == 'client') {
                if (!$server->isEstablished($k)) {
                    $table->del($k);
                    continue;
                }
                $server->push($k, json_encode(['action' => 'info', 'data' => $cacheData]));
            }
        }
    }

    public function deleteBlackIp()
    {
        $table = TableManager::getInstance()->get(K::TABLE_IP_400);
        $deleteKey = [];
        foreach ($table as $k => $v) {
            if (time() - $v['time'] > 15 * 60) $deleteKey[] = $k;
        }
        foreach ($deleteKey as $v) {
            $table->del($v);
            System::log($v, 'delete black ip');
        }
    }

    public function everySecond()
    {
        $configTable = TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG);
        $requestOk = $configTable->get(K::KEY_EVERY_SECOND_REQUEST_OK);
        if (!$requestOk) $requestOk = ['data' => '', 'number' => 0];
        $configTable->set(K::KEY_EVERY_SECOND_REQUEST_OK_CACHE, $requestOk);
        $requestAll = $configTable->get(K::KEY_EVERY_SECOND_REQUEST_ALL);
        if (!$requestAll) $requestAll = ['data' => '', 'number' => 0];
        $configTable->set(K::KEY_EVERY_SECOND_REQUEST_ALL_CACHE, $requestAll);
        $configTable->del(K::KEY_EVERY_SECOND_REQUEST_OK);
        $configTable->del(K::KEY_EVERY_SECOND_REQUEST_ALL);
        $currentTime = time();
        if ($currentTime % 5 != 0) return;
        //这里需要在0.3s之后再推送信息，尽量的保证子服务器把当前的信息推送上来
        Timer::getInstance()->after(300, [$this, 'sendData']);
//        $this->sendData();
        if ($currentTime % 60 === 0) $this->deleteBlackIp();
        $config = System::config();
        $postData = System::getServerInfo();
        //下面的这一段删除，需要放到  System::getServerInfo(); 之后，因为这个函数 会计算这个值
        if ($config[K::KEY_PROXY_HISTORY_ENABLE] && $config[K::KEY_PROXY_HISTORY]) {
            //启用了代理，那么要获取IP，每隔一段时间也要获取 IP
            if (!$config[K::KEY_PROXY_IP] || $currentTime % 1000 === 0) {
                $ip = Proxy::getProxyIp();
                $postData[K::KEY_PROXY_IP] = $ip;
                System::config(K::KEY_PROXY_IP, $ip);
//                if ($ip){
//                      //注意，这里不能使用if 来判断，因为如果刚刚启用了代理，并没有成功的获取代理的IP,如果判断了那么就是使用上次的IP，明显不对
//                }
            }
        } else {
            $postData[K::KEY_PROXY_IP] = 'no proxy';
        }
        try {
            $bigemapData = Config::getInstance()->getConf('BIGEMAP_DATA');
            if ($bigemapData['notice']) {
                SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/google/api/statistics', $postData, ['timeout' => 3, 'headers' => ['bigemap' => 1]]);
            }
        } catch (\Throwable $exception) {
            System::log($exception->getMessage(), 'checkTotalRequest error ' . date('Y-m-d H:i:s'));
        }
        $this->timerUploadPort();
        if (in_array($config[K::KEY_SERVER_ID], K::CONST_INNER_SERVER_ID)) {
            $this->checkSleepIndex();
        }
        //每隔半小时会向微信推送内容
        if ($currentTime % 1800 === 0) $this->noticeWechat();
        //每两个小时，会检查可能被封的服务器
        if ($currentTime % 7200 === 0) $this->checkForbiddenProxyServer();
        //检查当前的服务器主IP是不是已经解封了
        if ($currentTime % 600 === 0) $this->checkForbidden();
        //检查是不是已经封IP了，尝试自动拉代理
        if ($currentTime % 60 === 0) $this->autoStartProxy();
        //检查所有的子服务器是不是运行正常，因为目前不知道为什么服务器有时会死掉
        if ($currentTime % 600 === 0) $this->checkAllSubServer();
        if ($currentTime % 7200 === 0) {
            try {
                $this->checkOutDateProxyServer();
            }catch (\Throwable $throwable){
                echo 'check outdate proxy server error',$throwable->getMessage(),PHP_EOL;
            }
        }
    }

    public function checkOutDateProxyServer()
    {
        if (System::config(K::KEY_SERVER_PORT) != '9502') return;
        $host = parse_url(K::MANAGE_SERVER_ADDR, PHP_URL_HOST);
        $ip = gethostbyname($host);
        //如果不是主服务器的IP
        if (!$ip || $ip != System::config(K::KEY_CURRENT_IP)) return;
        //由于现在的代理有问题，无法模拟登陆了，所以需要手动的来判断有没有更新，有没有过期之类的
        $manager = new ProxyManager();
        $all = $manager->getAllServer();
        foreach ($all as $v) {
            if ($v['enable'] != 1) continue;
            if (time() - $v['resubscribe_time'] > 86400 * 0.9) {
                $manager->reSubscribeLink($v['id']);
            }

            if (time() - $v['deadline'] > -86400 * 0.8) {
                $noticeData['content'] = $v['title'];
                $noticeData['content'] .= "\r\n" . '过期时间：' . date('Y-m-d', $v['deadline']);
                $noticeData['action'] = 'subscribe_expire';
                WechatNotice::notice($noticeData);
            }
        }
    }

    public function checkSleepIndex()
    {
        $serverId = System::config(K::KEY_SERVER_ID);
        if (!in_array(trim($serverId), K::CONST_INNER_SERVER_ID)) return;
        $table = TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG);
        $data = $table->get(K::KEY_SLEEP_INDEX);
        $data['data'] = json_decode($data['data'], true);
        if (!$data['data']) {
            $data = [
                'data' => ['time' => time(), 'sleep' => mt_rand(20, 60 * 5)],
                'number' => 0,
            ];
            System::log('sleep index' . $data['number'] . ', next change time : ' . date('Y-m-d H:i:s', $data['data']['time'] + $data['data']['sleep']));
        }
        if (time() - $data['data']['time'] > $data['data']['sleep']) {
            $data['data'] = ['time' => time(), 'sleep' => mt_rand(20, 60 * 5)];
            $data['number']++;
            System::log('sleep index ' . $data['number'] . ',next change time : ' . date('Y-m-d H:i:s', $data['data']['time'] + $data['data']['sleep']));
        }
        $data['data'] = json_encode($data['data']);
        $table->set(K::KEY_SLEEP_INDEX, $data);
    }

    public function checkForbiddenProxyServer()
    {
        //如果不是正常的时间段，那么跳过
        if (date('H') >= 23 || date('H') < 8) return;
        //不是主服务的端口，那么跳过
        if (System::config(K::KEY_SERVER_PORT) != '9502') return;
        $host = parse_url(K::MANAGE_SERVER_ADDR, PHP_URL_HOST);
        $ip = gethostbyname($host);
        //如果不是主服务器的IP
        if (!$ip || $ip != System::config(K::KEY_CURRENT_IP)) return;
        $notice = ['action' => 'check_forbidden', 'content' => [], 'sum' => ['200' => 0, '403' => 0, 'else' => 0, 'jump' => 0]];
        $manager = ProxyManager::getInstance();
        $all = $manager->select('server_list', ['status' => 100]);
        //这里的所有分成了两个部分，一个部分是有IP的，另外的部分可能是没有IP的，先处理一下
        $ips = [];
        $parseData = [];
        foreach ($all as $v) {
            if ($v['ip']) {
                if (!isset($ips[$v['ip']])) {
                    $parseData[] = $v;
                    $ips[$v['ip']] = 1;
                }
            } else {
                $parseData[] = $v;
            }
        }
        if (!$parseData) {
            $notice['content'] = '所有服务器运行正常';
            WechatNotice::notice($notice);
            System::log('no forbidden proxy server');
            return;
        }
        $cachePath = System::config(K::KEY_CACHE_PATH);
        $logName = $cachePath . '/check_proxy_server_log';
        foreach ($parseData as $v) {
            $v2rayPath = $cachePath . '/v2ray/v2ray';
            if (!file_exists($v2rayPath)) {
                $status = Shell::getZipAndExtract('http://google.zhangwei.fun/xt/v2ray.zip', $cachePath . '/v2ray');
                if (!$status) {
                    System::log('check forbidden but unzip v2ray failed');
                    return false;
                }
            }

            $configTemplate = Proxy::getBaseTemplate();
            if (!$configTemplate) {
                System::log('check forbidden but get template failed');
                return false;
            }
            if (time() - $v['next_check_time'] <= 0) {
                $notice['sum']['jump']++;
                continue;
            }
            $testPort = 50008;
            $selected = json_decode($v['info'], true);
            if (!Proxy::startProxy($configTemplate, $selected, $testPort)) {
                file_put_contents($logName, date('Y-m-d H:i:s') . ' : start failed :' . json_encode($v) . PHP_EOL, FILE_APPEND);
                continue;
            }
            Coroutine::sleep(4);
            //启动成功了，那么进行一次测试
            try {
                $defaultData = GSession::getDefaultQueryData('second');
                SaberGM::post($defaultData['url'], $defaultData['data'], ['headers' => $defaultData['headers'], 'proxy' => 'socks5://127.0.0.1:' . $testPort]);
                file_put_contents($logName, date('Y-m-d H:i:s') . ' : check 200:' . json_encode($v) . PHP_EOL, FILE_APPEND);
                $code = 200;
            } catch (\Throwable $exception) {
                $code = $exception->getCode();
                file_put_contents($logName, date('Y-m-d H:i:s') . ' : check ' . $code . ':' . json_encode($v) . PHP_EOL, FILE_APPEND);
            }
            $currentIp = Proxy::getProxyIp('socks5://127.0.0.1:' . $testPort);
            //这里还有一种情况，由于某些代理只能在中国内地才能启用成功，如果判断到了，距离上次检查时间已经过了4天，那么直接启用这个代理
            if (time() - $v['next_check_time'] > 86400 * 3) $code = 200;
            switch ($code) {
                case 200:
                    $notice['sum']['200']++;
                    $updateData = ['status' => 1];
                    if ($currentIp) $updateData['ip'] = $currentIp;
                    $manager->save('server_list', ['key' => $v['key']], $updateData);
                    if ($v['ip']) {
                        $manager->save('server_list', ['ip' => $v['ip']], $updateData);
                    }
                    break;
                case 403:
                    $notice['sum']['403']++;
                    $updateData = ['next_check_time' => time() + 3600 * 8 + rand(3600, 3600 * 6)];
                    if ($currentIp) $updateData['ip'] = $currentIp;
                    $manager->save('server_list', ['key' => $v['key']], $updateData);
                    if ($v['ip']) {
                        $manager->save('server_list', ['ip' => $v['ip']], $updateData);
                    }
                    break;
                default:
                    $notice['sum']['else']++;
            }
        }
        $notice['content'][] = '当前共计最多被封 : ' . (count($parseData) - $notice['sum']['200']);
        $notice['content'][] = '解封 : ' . $notice['sum']['200'];
        $notice['content'][] = '封禁 : ' . $notice['sum']['403'];
        $notice['content'][] = '未通 : ' . $notice['sum']['else'];
        $notice['content'][] = '跳过 : ' . $notice['sum']['jump'];
        $notice['content'] = implode("\r\n", $notice['content']);
        WechatNotice::notice($notice);
    }

    public function noticeWechat()
    {
        //不是主服务的端口，那么跳过微信的通知
        if (System::config(K::KEY_SERVER_PORT) != '9502') return;
        $host = parse_url(K::MANAGE_SERVER_ADDR, PHP_URL_HOST);
        $ip = gethostbyname($host);
        //如果不是主服务器的IP
        if ($ip != System::config(K::KEY_CURRENT_IP)) return;
        $serverData = System::serverData();
        $notice = ['online' => 0, 'offline' => 0, 'day' => 0, 'proxy' => 0, 'enable' => 0];
        foreach ($serverData as $v) {
//            $dataKey=intval(str_replace(['history'],[''],$v[K::KEY_SERVER_ID]))>=1000?'inner':'outer';
            if ($v[K::KEY_EVERY_DAY_UPLOAD]['number']) $notice['day'] += $v[K::KEY_EVERY_DAY_UPLOAD]['number'];
            if ($v[K::KEY_PROXY_HISTORY_ENABLE]) $notice['proxy']++;
            if ($v[K::KEY_GOOGLE_HISTORY_ENABLE]) $notice['enable']++;
            if (time() - $v['time'] > 60) {
                $notice['offline']++;
            } else {
                $notice['online']++;
            }
        }
        $notice['sum'] = count($serverData);
        $notice['disabled'] = $notice['sum'] - $notice['enable'];
        $notice['day'] = round($notice['day'] / 1024 / 1024 / 1024, 2);
        $postData = ['action' => 'hour'];
        $postData['content'] = ['今日上传量:' . $notice['day'] . 'GB', '服务器总数:' . $notice['sum'], '服务器启用:' . $notice['enable'], '服务器在线:' . $notice['online'], '服务器离线:' . $notice['offline']];
        $postData['content'] = implode("\r\n", $postData['content']);
        WechatNotice::notice($postData);
    }

    public function updateIp()
    {
        $manager = ProxyManager::getInstance();
        $items = $manager->select('server_list', ['ip' => '']);
        $checkPort = 50007;
        $template = Proxy::getBaseTemplate();
        if (!$template) return false;
        foreach ($items as $v) {
            $config = json_decode($v['info'], true);
            if (!$config) continue;
            Proxy::startProxy($template, $config, $checkPort);
            Coroutine::sleep(4);
            $ip = Proxy::getProxyIp('socks5://127.0.0.1:' . $checkPort);
            if ($ip) {
                $manager->save('server_list', ['id' => $v['id']], ['ip' => $ip]);
            }
        }
    }

    public function checkAllSubServer()
    {
        $systemConfig = System::config();
//        if (!$systemConfig[K::KEY_SERVER_CHECK_ENABLE]) return;
        //只有主服务器才会检测
        if ($systemConfig[K::KEY_SERVER_ID] != K::CONST_MANAGER_SERVER_ID) return;
        $host = parse_url(K::MANAGE_SERVER_ADDR, PHP_URL_HOST);
        $ip = gethostbyname($host);
        try {
            $currentIp = SaberGM::get(K::MANAGE_SERVER_ADDR_9502 . '/google/index/ip', ['timeout' => 3]);
            $currentIp = trim($currentIp->getBody()->getContents());
            if (!$currentIp) return;
            //更新此服务器的IP
            System::config(K::KEY_CURRENT_IP, $currentIp);
            //只有主服务器上面才会进行判断
            if ($currentIp != $ip) return;
        } catch (\Throwable $exception) {
            System::log('get local ip failed');
            return;
        }
        $cachePath = $systemConfig[K::KEY_CACHE_PATH];
        $serverItems = System::serverData();
        for ($i = 0; $i < 3; $i++) {
            Coroutine::sleep(3);
            //这里要连续的查询三次，只有三次都失败的才会重启
            foreach ($serverItems as $k => $v) {
                //如果没有IP，没有服务器的ID
                if (!$v['ip'] || !$v[K::KEY_SERVER_ID]) continue;
                //判断当前的内网服务器，有没有旧的IP，因为内网的服务器，每隔一段时间，会换IP，那么到时旧的IP要删除掉
                $serverIds[$v[K::KEY_SERVER_ID]][] = $k;
                $serverIds[$v[K::KEY_SERVER_ID]] = array_unique($serverIds[$v[K::KEY_SERVER_ID]]);
                //如果已经检查过一次了，并且是服务器是连通的，那么不需要检查 这个服务器了;
                if ($i > 0 && !$v['__offline']) continue;
                try {
                    SaberGM::get($v['ip'] . ':' . $v[K::KEY_SERVER_PORT], ['timeout' => 3]);
                } catch (\Throwable $exception) {
                    //出错了，记录一次
                    file_put_contents($cachePath . '/check_sub_server', $v[K::KEY_SERVER_ID] . ' =>' . $exception->getCode() . PHP_EOL, FILE_APPEND);
                    Coroutine::sleep(1);
                    switch ($exception->getCode()) {
                        case -1:
                        case -2:
                        case -3:
                            //如果服务死掉了，返回的状态码是小于0的
                            if (!$v['__offline']) $serverItems[$k]['__offline'] = 0;
                            $serverItems[$k]['__offline']++;
                            break;
                    }
                }
            }
        }
        //经过上面的检查，判断出哪些是需要重启的
        $restart = [];
        $noticeServer = [
            'restart' => ['content' => [], 'action' => 'restart'],
            'server_offline' => ['content' => [], 'action' => 'server_offline'],
            'server_disable' => ['content' => [], 'action' => 'server_disable'],
            'server_low_traffic' => ['content' => [], 'action' => 'server_low_traffic'],
            'server_error' => ['content' => [], 'action' => 'server_error'],
        ];
        $oldTrafficData = file_get_contents($cachePath . '/traffic_data');
        $oldTrafficData = json_decode($oldTrafficData, true);
        if (!$oldTrafficData) $oldTrafficData = [];
        foreach ($serverItems as $k => $v) {
            //如果是内网的掉线，同时，有重名的服务器ID，那么要删除掉
            if (System::getServerIdNumber($v[K::KEY_SERVER_ID]) >= 1000 && count($serverIds[$v[K::KEY_SERVER_ID]]) > 1 && time() - $v['time'] > 60) {
                unlink($cachePath . '/ip/' . $k . '.ip');
                unset($oldTrafficData[$k]);
                //由于这个是属性过期的服务器，那么不需要继续进行下面的判断
                continue;
            }
            if (!$v[K::KEY_GOOGLE_HISTORY_ENABLE]) {
                $noticeServer['server_disable']['content'][] = $v;
            }
            if (time() - $v['time'] > 600) {
                $noticeServer['server_offline']['content'][] = $v;
            }

            if (time() - $v['time'] < 120 && !$v[K::KEY_GOOGLE_HISTORY_ENABLE] && $v[K::KEY_PROXY_HISTORY_ENABLE] && time() - $v[K::KEY_GOOGLE_HISTORY_DISABLED_TIME] > 60 * 10) {
                //如果在2分钟之后提交过数据，同时是禁用的，同时是启用的代理，同时停止已经超过了 10分钟，那么是很有可能代理没有启动成功
                $noticeServer['server_error']['content'][] = $v;
            }

            if (intval($v[K::KEY_EVERY_DAY_UPLOAD]['number']) - intval($oldTrafficData[$k][K::KEY_EVERY_DAY_UPLOAD]['number']) < 10) {
                //如果在10分钟之内没有流星的增加，那么认为这个服务器上面的流量有些问题
                //由于后来我们只使用了内网中的服务器，外网的服务器流量很少，所以这里暂停检查 外网的服务器
                if (System::getServerIdNumber($v[K::KEY_SERVER_ID]) >= 1000) {
                    $noticeServer['server_low_traffic']['content'][] = $v;
                }
            }
            $oldTrafficData[$k] = $v;
//            if ((date('H') > 12 && $v[K::KEY_EVERY_DAY_UPLOAD]['number'] < 1024 * 1024*100||date('H') >=8 && $v[K::KEY_EVERY_DAY_UPLOAD]['number'] < 1024*1024 * 5) && $v[K::KEY_GOOGLE_HISTORY_ENABLE]&&$v[K::KEY_SERVER_ID]) {
//                //每天8点之后，流量小于5M的，每天12点之后，流量还是少于100M的并且启用的服务器，表示异常
//                $noticeServer['server_low_traffic']['content'][] = $v;
//            }
            //断线少于3次，或者是我们自己的服务器端口号不是9502的。不重启（因为我们自己的服务器内网没有开放端口）
            if ($v['__offline'] < 3 || ($v[K::KEY_SERVER_PORT] != 9502 && intval(str_replace(['history'], [''], $v[K::KEY_SERVER_ID])) > 1000)) continue;
            if (!$restart[$v['ip']]) $restart[$v['ip']] = [];
            $restart[$v['ip']][] = $v[K::KEY_SERVER_PORT];
            $noticeServer['restart']['content'][] = $v[K::KEY_SERVER_ID];
        }
        //只要共同的服务器数据
        $oldTrafficData = array_intersect_key($oldTrafficData, $serverItems);
        file_put_contents($cachePath . '/traffic_data', json_encode($oldTrafficData));
        if ($noticeServer['restart']['content']) {
            $noticeServer['restart']['content'] = implode(',', $noticeServer['restart']['content']);
            WechatNotice::notice($noticeServer['restart']);
        }
        if (count($noticeServer['server_disable']['content']) >= 10) {
            $noticeServer['server_disable']['content'] = '当前共有' . count($noticeServer['server_disable']['content']) . '个服务器停止了工作';
            WechatNotice::notice($noticeServer['server_disable']);
        }
        if (count($noticeServer['server_offline']['content']) >= 2) {
            $noticeServer['server_offline']['content'] = '当前共有' . count($noticeServer['server_offline']['content']) . '个服务器掉线';
            WechatNotice::notice($noticeServer['server_offline']);
        }

        if (count($noticeServer['server_low_traffic']['content']) >= 1) {
            if (count($noticeServer['server_low_traffic']['content']) <= 5) {
                //如果有5个以内的服务器流量异常，要展示对应的服务器ID
                $lowTrafficIds = array_map(function ($v) {
                    return System::getServerIdNumber($v[K::KEY_SERVER_ID]);
                }, $noticeServer['server_low_traffic']['content']);
                $noticeServer['server_low_traffic']['content'] = '服务器:' . implode(',', $lowTrafficIds) . ' 流量异常';
            } else {
                $noticeServer['server_low_traffic']['content'] = '当前共有 ' . count($noticeServer['server_low_traffic']['content']) . ' 个服务器流量过低';
            }
            WechatNotice::notice($noticeServer['server_low_traffic']);
        }
        if (count($noticeServer['server_error']['content']) >= 1) {
            $noticeServer['server_error']['content'] = '当前共有' . count($noticeServer['server_error']['content']) . '个服务器异常';
            WechatNotice::notice($noticeServer['server_error']);
        }
        if ($restart) System::log($restart, 'restart server list');
        foreach ($restart as $k => $v) {
            //如果有9502需要重启，那么只需要重启这个9502，因为9502端口重启的话。会默认重启下面的子服务器
            try {
                if (in_array(9502, $v)) {
                    SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/google/api/server?bigemap=1', ['action' => 'start-server', 'ip' => $k, K::KEY_SERVER_PORT => 9502], ['timeout' => 2]);
                } else {
                    foreach ($v as $vv) {
                        SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/google/api/server?bigemap=1', ['action' => 'start-server', 'ip' => $k, K::KEY_SERVER_PORT => $vv], ['timeout' => 2]);
                    }
                }
            } catch (\Throwable $exception) {
                System::log('restart stop server failed');
            }
        }
    }

    public function checkProxyServer()
    {
        $config = System::config();
        //检查当前的代理服务器有没有重复
        try {
            $result = SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/google/api/proxy', ['code' => 200, 'result' => $config], ['timeout' => 3]);
            $result = $result->getBody()->getContents();
            $result = json_decode($result, true);
            switch ($result['data']['action']) {
                case 'used':
                    //当前使用的代理 已经被别人使用了，那么要重新拉取一次
                    System::log(System::config(K::KEY_PROXY_CURRENT), 'rebuild proxy server because used,current info');
                    System::log($result['data'], 'rebuild proxy server because used,in use info');
                    //清空代理的IP信息
                    System::config(K::KEY_PROXY_IP, '');
                    System::config(K::KEY_GOOGLE_HISTORY_ENABLE, '');
                    break;
            }
        } catch (\Throwable $exception) {
            System::log($exception->getMessage(), 'check proxy server is repeat failed');
        }
    }

    public function autoStartProxy()
    {
        $config = System::config();
        if (!$config[K::KEY_SERVER_ID]) {
            System::log('no server id no create proxy');
            return;
        }
        if ($config[K::KEY_GOOGLE_HISTORY_ENABLE]) {
            //已经使用了开启了代理服务器，
            //那么，那么要告诉主服务器，有没有在使用代理 ，如果没有，则清空当前的引用，如果有，判断一下有没有相同的IP，或者是重复的代理 ，否则会更新代理
            //由于sqlite数据库有锁的机制 ，为了最大限度 的保证不会出错 ，这里使用了随机的延迟;
            $delta = mt_rand(10, 59000);
            Timer::getInstance()->after($delta, [$this, 'checkProxyServer']);
            return;
        }
        if ($config[K::KEY_SERVER_PORT] != 9502 && $config[K::KEY_PARENT_IP]) {
            //如果不是主端口，同时有父级ID,那么由于服务器性能低了，同时只允许一部分的子服务器检查代理
            try {
                if (intval(str_replace(['history'], [''], $config[K::KEY_SERVER_ID])) > 1000) {
                    $canGo = SaberGM::post('http://127.0.0.1:9502/google/server/proxyCheck', ['port' => $config[K::KEY_SERVER_PORT]], ['timeout' => 3]);
                } else {
                    $canGo = SaberGM::post($config[K::KEY_PARENT_IP] . '/google/server/proxyCheck', ['port' => $config[K::KEY_SERVER_PORT]], ['timeout' => 3]);
                }
                $canGo = $canGo->getBody()->getContents();
                $canGo = json_decode($canGo, true);
                if ($canGo['code'] != 200) {
                    System::log($canGo, 'server response');
                    return;
                }
            } catch (\Throwable $throwable) {
                System::log($throwable->getMessage(), 'check is start proxy ok?');
                return;
            }
        }
        //当前的服务器已经被封了，从主服务器上面拉取全部的代理服务器，尝试开启代理
        $proxyServer = Proxy::getAllProxy();
        if (!$proxyServer) {
            System::log('get proxy server from main server failed');
            return;
        }
        $unusedServer = [];
//        $location = $config[K::KEY_GOOGLE_HISTORY_LOCATION];
        $serverId = System::getServerIdNumber();
        foreach ($proxyServer as $v) {
            if ($serverId < 1000) {
                //这个是我们外网的服务器,不可以使用内网的专用服务器
                if ($v['type'] == '2') continue;
            } else {
                //这个是内网的服务器，不可以使用外网的专用服务器
                if ($v['type'] == '1') continue;
            }
            foreach ($v['children'] as $vv) {
                //如果没有启用，或者启用了，但是更新时间在10分钟之前的，或者是被封了，但是上次检查时间已经过了指定的时间
//                    || ($vv['status'] == 100 && $vv['next_check_time'] < time())
                if ($vv['status'] == 1 || ($vv['status'] == 2 && time() - $vv['update_time'] > 60 * 10)) {
//                    $info = json_decode($vv['info']);
                    //保存当前的类型，方便后面的判断
                    $vv['_type_'] = $v['type'];
                    $unusedServer[] = $vv;
                }
            }
        }
        if (!$unusedServer) {
            System::log('no more proxy server to use');
            return;
        }
        $v2rayPath=$config[K::KEY_CACHE_PATH].'/clash/clash';
        if (!file_exists($v2rayPath)) {
            $status = Shell::getZipAndExtract(K::MANAGE_SERVER_ADDR.'/xt/clash.zip', $config[K::KEY_CACHE_PATH] . '/clash');
            if (!$status) {
                System::log('check forbidden but unzip v2ray failed');
                return false;
            }
        }
        //随机一次，减少选择了重复的代理的情况
        shuffle($unusedServer);
        $unusedServer = array_values($unusedServer);
        $parseData = [];
        foreach ($unusedServer as $k => $v) {
            $place = json_decode($v['info'], true);
            if (($serverId >= 1000 && $v['_type_'] == '2')) {
                //如果是内网的专用服务器,那么要排在前面;
                $parseData[$k] = $v;
//                array_unshift($parseData[1], $v);
            } else if ($serverId < 1000 && strpos($place['ps'], '港') !== false && mt_rand(1, 10) > 5) {
                //如果是外网的服务器,名称中包含了香港关键字并且有一定的机会,排在前面
                $parseData[$k] = $v;
            } else if ($serverId < 1000 && Proxy::checkIsPrimary($place['ps'])) {
                //如果是外网的服务器,名称中包含了其它关键字,排在第二位
                $kk = $k + 5000;
                $parseData[$kk] = $v;
            } else {
                $kk = $k + 10000;
                $parseData[$kk] = $v;
            }
        }
        ksort($parseData);
        $result = [];
        $defaultData = GSession::getDefaultQueryData('second');
        $proxyPort = $config[K::KEY_PROXY_PORT];
        if (!$proxyPort) $proxyPort = 50010;
        $template = Proxy::getBaseTemplate();
        if (!$template) {
            System::log('auto start proxy ,but no template');
            return;
        }
        $configTable = TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG);
        if ($configTable->exist(K::KEY_PROXY_CHECK_IS_RUNNING)) {
            System::log('proxy check is running wait for next check');
            return;
        }
        System::log('start to choose proxy server ,all server count:' . count($parseData));
        $configTable->set(K::KEY_PROXY_CHECK_IS_RUNNING, ['data' => 1, 'number' => 1]);
        foreach ($parseData as $v) {
            $info = json_decode($v['info'], true);
            if (!$info) continue;
            Coroutine::sleep(1);
            if (!Proxy::startProxy($template, $info, $proxyPort)) continue;
            Coroutine::sleep(4);
            try {
                $proxyUrl = 'socks5://127.0.0.1:' . $proxyPort;
                $checkSession = SaberGM::post($defaultData['url'], $defaultData['data'], ['proxy' => $proxyUrl, 'timeout' => 2, 'headers' => $defaultData['headers']]);
                //说明当前的这个代理是可用的，那么设置好当前的代理信息
                System::config(K::KEY_PROXY_HISTORY, $proxyUrl);
                System::config(K::KEY_PROXY_HISTORY_ENABLE, '1');
                System::config(K::KEY_GOOGLE_HISTORY_ENABLE, 1);
                //保存当前的代理的KEY值
                System::config(K::KEY_PROXY_CURRENT, [$v['pid'], $v['key']]);
                //把当前的这个Session 保存起来
                GSession::saveSession($checkSession->getBody()->getContents());
                $ip = Proxy::getProxyIp($proxyUrl);
                if ($ip) {
                    System::config(K::KEY_PROXY_IP, $ip);
                    $v['ip'] = $ip;
                }
                System::log($v, 'start proxy server ok,current ip: ' . $ip);
                //退出查询
                break;
            } catch (\Throwable $exception) {
                $code = $exception->getCode();
                switch ($code) {
                    case 403:
                        //如果是403，那么需要告诉主服务器，这样在主服务器上面会定时检查这个代理的可用性
                        $v['code'] = 403;
                        $result[] = $v;
                        break;
                }
            }
        }
        $configTable->del(K::KEY_PROXY_CHECK_IS_RUNNING);
        //然后把本次可能出现检查结果为403的服务器告诉主服务器，
        if (!$result) return;
        try {
            SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/google/api/proxy', ['code' => 403, 'result' => $result], ['timeout' => 3]);
        } catch (\Throwable $exception) {

        }
    }

    private function checkForbidden()
    {
        $config = System::config();
        $mainConfig = Config::getInstance()->getConf('MAIN_SERVER');
        if ($mainConfig['PORT'] == '9502') {
            //如果是主程序的端口，那么在启动了代理，或者是没有启用历史影像的时候，需要检查当前的服务器是不是被解封了，
            if (!$config[K::KEY_GOOGLE_HISTORY_ENABLE] || ($config[K::KEY_PROXY_HISTORY_ENABLE] && $config[K::KEY_PROXY_HISTORY])) {
                //当前谷歌历史被封了
                $lastCheckTime = System::config(K::KEY_GOOGLE_HISTORY_FORBIDDEN_LAST_CHECK_TIME);
                if (time() > $lastCheckTime) {
                    //距离上次检查已经过了6个小时了，那么检查一次
                    try {
                        $defaultData = GSession::getDefaultQueryData('second');
//                    $saber=System::getSaber(false);
                        $saber = Saber::create([
                            'base_uri' => 'https://kh.google.com',
                        ]);
                        $testData = $saber->post($defaultData['url'], $defaultData['data'], ['headers' => $defaultData['headers']]);
                        $testSession = $testData->getBody()->getContents();
                        GSession::saveSession($testSession);
                        file_put_contents($config[K::KEY_CACHE_PATH] . '/check_forbidden', date('Y-m-d H:i:s') . PHP_EOL . base64_encode($testSession) . PHP_EOL, FILE_APPEND);
                        //说明当前可以访问，那么要启用谷歌历史影像
                        System::config(K::KEY_GOOGLE_HISTORY_ENABLE, '1');
                        //关闭代理
                        System::config(K::KEY_PROXY_HISTORY_ENABLE, '');
                    } catch (\Throwable $exception) {
                        file_put_contents($config[K::KEY_CACHE_PATH] . '/check_forbidden', date('Y-m-d H:i:s') . PHP_EOL . $exception->getMessage() . PHP_EOL, FILE_APPEND);
                    }
                    //下次的检查时间至少为6个小时
                    $nextTime = time() + 3600 * 6 + mt_rand(0, 3600 * 4);
                    System::config(K::KEY_GOOGLE_HISTORY_FORBIDDEN_LAST_CHECK_TIME, $nextTime);
                    file_put_contents($config[K::KEY_CACHE_PATH] . '/check_forbidden', 'next check time' . PHP_EOL . date('Y-m-d H:i:s', $nextTime) . PHP_EOL, FILE_APPEND);
                }
            }
        } else {
            //不是主程序的端口，那么必须要使用代理
            if (!$config[K::KEY_PROXY_HISTORY_ENABLE] || !$config[K::KEY_PROXY_HISTORY]) {
                //如果没有启动代理 ，或者是没有代理的地址，那么当前的这个服务，需要停止
                System::config(K::KEY_GOOGLE_HISTORY_ENABLE, '');
            }
        }
    }

    public function curlSession()
    {
        $firstTable = GSession::getTable('first');
        $secondTable = GSession::getTable('second');
        $delKey = [];
        foreach ($firstTable as $k => $v) {
            if (time() - $v['time'] > 60 * 60 * 2) {
                $delKey[] = $k;
            }
        }
        foreach ($delKey as $v) {
            $firstTable->del($v);
        }
        $delKey = [];
        $numberServerId = System::getServerIdNumber();
//        if ($numberServerId>=1000&&(date('H')>22||date('H')<5)){
//            //如果是内网的服务器，在每天的晚上11点之后，早上的5点前，不会产生session,反一直返回 403 ，这样是为了防止被封
//            return;
//        }
        foreach ($secondTable as $k => $v) {
            //只有外网的浏览服务器才会删除过期的session 内网中由于会休息，为了防止服务器在休息的时候产生session
//            $numberServerId<1000&&
            if (time() - $v['time'] >= 590) {
                $delKey[] = $k;
            }
        }

        foreach ($delKey as $v) {
            $secondTable->del($v);
        }
        $delKey = [];
        $queryTable = TableManager::getInstance()->get(K::TABLE_QUERY_LIST);
        $configTable = TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG);
        $replaceData = [];
        foreach ($queryTable as $k => $v) {
            if (!$v['first_data'] && !$v['second_data']) {
                $delKey[] = $k;
                continue;
            }
            $lastQueryTime = $configTable->get(K::KEY_FIRST_LAST_QUERY_TIME);
            if ($v['first_data'] && $v['first_header'] && $firstTable->count() < 3 && time() - $lastQueryTime['number'] > 0 && !$configTable->exist(K::KEY_IS_FIRST_QUERY_RUNNING)) {
                $configTable->set(K::KEY_IS_FIRST_QUERY_RUNNING, ['data' => '1', 'number' => 1]);
                $delKey[] = $k;
                try {
                    $saber = System::getSaber();
                    $data = $saber->post('https://kh.google.com/geauth', base64_decode($v['first_data']), ['headers' => json_decode($v['first_header'], true)]);
                    $configTable->set(K::KEY_FIRST_LAST_QUERY_TIME, ['data' => '1', 'number' => \time() + mt_rand(300, 900)]);
                    $content = $data->getBody()->getContents();
                    $firstTable->set(uniqid(), ['data' => base64_encode($content), 'sum' => 0, 'time' => time()]);
                    System::log(base64_encode($content), 'first curl data ip  ' . $k);
                } catch (\Throwable $exception) {
                    if (!$firstTable->count()) {
                        //如果没有长度的情况下，同时又报错了，增加一次报警
                        if (System::incr(K::KEY_GOOGLE_HISTORY_FORBIDDEN_TIMES) > 15 && System::config(K::KEY_GOOGLE_HISTORY_ENABLE)) {
                            System::log('first session more than 15 times');
                            System::config(K::KEY_GOOGLE_HISTORY_DISABLED_TIME, time());
                            System::config(K::KEY_GOOGLE_HISTORY_ENABLE, '');
                        }
                    }
                    $configTable->set(K::KEY_FIRST_LAST_QUERY_TIME, ['data' => '1', 'number' => \time() + mt_rand(2, 10)]);
                    System::log($exception->getMessage(), 'first curl error ip ' . $k);
                }
                $configTable->del(K::KEY_IS_FIRST_QUERY_RUNNING);
            }
            $secondLastQuery = $configTable->get(K::KEY_SECOND_LAST_QUERY_FIRST_TIME);
            if ($v['second_data'] && $v['second_header'] && $secondTable->count() < 2 && \time() - $secondLastQuery['number'] > 0 && !$configTable->exist(K::KEY_IS_SECOND_QUERY_RUNNING)) {
                $delKey[] = $k;
                $configTable->set(K::KEY_IS_SECOND_QUERY_RUNNING, ['data' => '1', 'number' => 1]);
                try {
                    $fixed = System::config(K::KEY_FIXED_SECOND_QUERY);
                    if (!$secondTable->count() || $fixed) {
                        System::log('second query form default data');
                        $defaultData = GSession::getDefaultQueryData('second');
                        $v['second_header'] = json_encode($defaultData['headers'], JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
                        $v['second_data'] = base64_encode($defaultData['data']);
                    }
                    $saber = System::getSaber();
                    $data = $saber->post('https://kh.google.com/geauth?ct=free', base64_decode($v['second_data']), ['headers' => json_decode($v['second_header'], true)]);
                    $configTable->set(K::KEY_SECOND_LAST_QUERY_FIRST_TIME, ['data' => 1, 'number' => \time() + mt_rand(420, 540)]);
                    $content = $data->getBody()->getContents();
                    if ($k != '127.0.0.1' && !$fixed && $secondTable->count() && strpos($v['second_data'], 'dHXKm3Wtlu57ZMFpUPvooLznsECUCpMXgZfgALI5IuY1') === false) {
                        //只有有长度了,，才不是使用默认的数据来生成的，也不是默认客户端提交的,那么保存这个数据
                        //AQAAAAM3vfBl6/dHXKm3Wtlu57ZMFpUPvooLznsECUCpMXgZfgALI5IuY1Ydw7DmtQ== 客户端的默认数据
                        $cachePath = System::config(K::KEY_CACHE_PATH);
                        file_put_contents($cachePath . '/session-ok', $v['second_data'] . PHP_EOL, FILE_APPEND);
                    }
                    System::log('code 200  ip ' . $k . ' ' . $v['second_data'], 'second curl ok ');
                    GSession::saveSession($content);
                    $configTable->del(K::KEY_GOOGLE_HISTORY_FORBIDDEN_TIMES);
                } catch (\Throwable $exception) {
                    $errorCode = $exception->getCode();
                    if ($errorCode == 403) {
                        //如果是403错误，那么要报告主服务器，将当前可能使用的代理停止使用,
                        $systemConfig = System::config();
                        if ($systemConfig[K::KEY_PROXY_HISTORY_ENABLE] && $systemConfig[K::KEY_PROXY_CURRENT]) {
                            Proxy::noticeForbiddenServer($systemConfig[K::KEY_PROXY_CURRENT][1]);
                        }
                    }
                    if (!$secondTable->count() && System::config(K::KEY_GOOGLE_HISTORY_ENABLE) || $errorCode == 403) {
                        //如果没有长度的情况下，同时又报错了，增加一次报警,,超过20次停止服务器，如果是403错误，直接停止服务器，
                        if (System::incr(K::KEY_GOOGLE_HISTORY_FORBIDDEN_TIMES) > 15 || $errorCode == 403) {
                            System::log('second session more than 15 times or 403 error');
                            System::config(K::KEY_GOOGLE_HISTORY_DISABLED_TIME, time());
                            System::config(K::KEY_GOOGLE_HISTORY_ENABLE, '');
                        }
                    }
                    //如果失败了，那么也需要等待10秒后再重试
                    $configTable->set(K::KEY_SECOND_LAST_QUERY_FIRST_TIME, ['data' => 1, 'number' => \time() + 10]);
                    System::log('code  ' . $exception->getCode() . ' ip   ' . $k . '   ' . $v['second_data'], 'second curl error');
                }
                $configTable->del(K::KEY_IS_SECOND_QUERY_RUNNING);
            }
        }

        foreach ($delKey as $v) {
            $queryTable->del($v);
        }
    }

    public function timerUploadPort()
    {
        $config = System::config();
        $bigemapData = Config::getInstance()->getConf('BIGEMAP_DATA');
        $serverId = str_replace(['history'], [''], $config[K::KEY_SERVER_ID]);
        $serverId = intval(trim($serverId));
        if ($bigemapData['notice'] && $config[K::KEY_SERVER_ID] && $config[K::KEY_GOOGLE_HISTORY_ENABLE]) {
            //本地不会上传IP地址，服务器的IP都是172开头的
            switch (true) {
                case !empty($config[K::KEY_PARENT_IP]):
                    //有父级ID，那么要向这个父级服务器来提交数据
                    if ($serverId > 1000) {
                        $url = 'http://127.0.0.1:9502/google/api/sub';
                    } else {
                        $url = $config[K::KEY_PARENT_IP] . '/google/api/sub';
                    }
                    $data = [
                        'port' => $config[K::KEY_SERVER_PORT],
                    ];
                    try {
                        SaberGM::post($url, $data, ['timeout' => 3]);
                    } catch (\Throwable $exception) {

                    }
                    break;
                case !empty($config[K::KEY_SERVER_ID])&&$bigemapData['noticeLuo']:
                    //如果有服务器的名称，那么要向罗总的服务器提交数据,后来由于在内网中要添加服务器，下载谷歌的数据，这里认为只要是id >=1000 的服务器，也不再提交
                    if ($serverId < 1000) {
                        try {
//                            SaberGM::get('http://kh.bigemap.com/localip.php?data=' . $config[K::KEY_SERVER_ID] . ':' . implode('|', [$config[K::KEY_SERVER_PORT], $bigemapData['type']]), ['timeout' => 3]);
                        } catch (\Throwable $exception) {

                        }
                    }
                    break;
            }
        }
    }

}
