<?php

namespace App\HttpController\Google;

use AlibabaCloud\Cloudauth\V20190307\InitFaceVerify;
use App\Key\K;
use App\Tools\Shell;
use App\Tools\System;
use EasySwoole\Component\TableManager;
use Swoole\Coroutine;

class Server extends Base
{
    public function index()
    {
        $data = $this->getRequestData();
        switch ($data['action']) {
            case 'create-server':
                //创建一个服务器
                $this->createServer();
                break;
            case 'update-server':
                //更新一个服务器
                $this->updateServer();
                break;
            case 'start-server':
                Shell::shellKill('bigemap'.$data[K::KEY_SERVER_PORT]);
                $serverDir=dirname(EASYSWOOLE_ROOT).'/easyswoole'.$data[K::KEY_SERVER_PORT];
                if (!is_dir($serverDir)) return $this->jsonError('没有找到程序的文件夹!');
                Shell::shellStart('php '.$serverDir.'/easyswoole server start -d -mode=produce');
                return $this->jsonSuccess('操作成功!');
                break;
            case 'stop-server':
                Shell::shellKill('bigemap'.$data[K::KEY_SERVER_PORT]);
                return $this->jsonSuccess('操作成功!');
                break;
            default:
                return $this->jsonError('未知操作');
        }
    }

    private function createServer()
    {
        $data = $this->getRequestData();
        if (!$data[K::KEY_SERVER_PORT] || !$data[K::KEY_PROXY_PORT]) {
            return $this->jsonError('没有提交端口号');
        }
        //检查有没有冲突的情况
        $hasProcess=Shell::checkProcess('bigemap'.$data[K::KEY_SERVER_PORT]);
        if ($hasProcess)return $this->jsonError('有进程占用!');
        $allDir=scandir(dirname(EASYSWOOLE_ROOT));
        if (in_array('easyswoole'.$data[K::KEY_SERVER_PORT],$allDir)) return  $this->jsonError('已经有此文件夹存在');
        $cacheDir=System::config(K::KEY_CACHE_PATH);
        $tmpDir=$cacheDir.'/tmp';
        if (!$tmpDir&&!mkdir($tmpDir)) return $this->jsonError('创建临时文件夹失败');
        $allCache=scandir(dirname($cacheDir));
        if (in_array($data[K::KEY_SERVER_PORT],$allCache)) return $this->jsonError('缓存文件夹被占用');
        //检查是不是有代理被占用，扫描所有的缓存文件夹，找到对应的配置文件
        foreach ($allCache as $v){
            if (is_numeric($v)){
                //如果是数字文件夹，那么认为是一个服务的缓存文件夹，
                $config=file_get_contents(dirname($cacheDir).'/'.$v.'/config');
                if ($config){
                    $config=json_decode($config,true);
                    if (!$config) return  $this->jsonError('解析旧配置文件出错');
                    if ($config[K::KEY_PROXY_PORT]==$data[K::KEY_PROXY_PORT]){
                        return  $this->jsonError('已经有代理使用-'.$config[K::KEY_SERVER_ID]);
                    }
                    if ($config[K::KEY_SERVER_PORT]==$data[K::KEY_SERVER_PORT]) {
                        return  $this->jsonError('已经有服务使用-'.$config[K::KEY_SERVER_ID]);
                    }
                }
            }
        }
        //没有端口被占用，也没有代理的端口被占用，那么下载主程序的文件，和配置文件
        $serverZip=K::MANAGE_SERVER_ADDR.'/xt/easyswoole.zip';
        $serverDir=dirname(EASYSWOOLE_ROOT).'/easyswoole'.$data[K::KEY_SERVER_PORT];
        $tmpServerDir=$tmpDir.'/easyswoole-'.date('Y-m-d-H-i-s');
        $saveStatus=Shell::getZipAndExtract($serverZip,$tmpServerDir);
        if (!$saveStatus) return $this->jsonError('下载主程序失败!');

        //然后把端口修改了
        $configFile=$tmpServerDir.'/easyswoole/produce.php';
        if (!is_file($configFile)) return  $this->jsonError('没有找到配置文件');
        $baseConfig=file_get_contents($configFile);
        if (in_array(System::config(K::KEY_SERVER_ID),K::CONST_INNER_SERVER_ID)){
            //如果是内网中的服务器，
            $baseConfig=str_replace(['9402','9502','250000','swoole_cpu_num()*2'],[$data[K::KEY_SERVER_PORT],$data[K::KEY_SERVER_PORT],'250000',1],$baseConfig);
        }else{
            $baseConfig=str_replace(['9402','9502','250000','swoole_cpu_num()*2'],[$data[K::KEY_SERVER_PORT],$data[K::KEY_SERVER_PORT],'150000',1],$baseConfig);
        }
        if (!$baseConfig) return $this->jsonError('配置文件替换失败!');
        file_put_contents($configFile,$baseConfig);

        if (!Shell::shellStart('mv '.$tmpServerDir.'/easyswoole '.$serverDir)) return $this->jsonError('移动主程序失败!');

        $configDir=dirname($cacheDir).'/'.$data[K::KEY_SERVER_PORT];
        $tmpConfigDir=$tmpDir.'/cache-'.date('Y-m-d-H-i-s');
        if (!Shell::getZipAndExtract(K::MANAGE_SERVER_ADDR.'/xt/cache.zip',$tmpConfigDir)) return $this->jsonError('下载配置文件失败!');
        //最后这里要注意一下，要保存代理的端口号信息,和默认停用此服务器，方便程序自动开启代理信息
//        echo $tmpConfigDir.'/cache/config',PHP_EOL;
//        var_dump(is_dir($tmpConfigDir.'/cache'));
        if (!file_put_contents($tmpConfigDir.'/cache/config',json_encode([K::KEY_GOOGLE_HISTORY_ENABLE=>'',K::KEY_PROXY_PORT=>$data[K::KEY_PROXY_PORT]]))) return $this->jsonError('保存代理端口号失败!');

        if (!Shell::shellStart('mv '.$tmpConfigDir.'/cache '.$configDir)) return $this->jsonError('移动配置文件夹失败!');
        //然后启动程序
        if (!Shell::shellStart('php '.$serverDir.'/easyswoole server start -d -mode=produce')) return $this->jsonError('启动程序失败!');
        return  $this->jsonSuccess('添加成功!');
    }

    private function updateServer(){
        $cacheDir=System::config(K::KEY_CACHE_PATH);
        $tmpCacheDir=$cacheDir.'/tmp';
        if (!is_dir($tmpCacheDir)&&!mkdir($tmpCacheDir)) return $this->jsonError('创建临时文件夹失败');
        $data=$this->getRequestData();
        if (!$data[K::KEY_SERVER_PORT]) return $this->jsonError('未知端口');
        $serverDir=dirname(EASYSWOOLE_ROOT).'/easyswoole'.$data[K::KEY_SERVER_PORT];
//        if (!is_dir($serverDir)) return $this->jsonError('没有找到要升级的服务器');
        $tmpServerDir=$tmpCacheDir.'/easyswoole-'.date('Y-m-d-H-i-s');
        if (!Shell::getZipAndExtract(K::MANAGE_SERVER_ADDR.'/xt/easyswoole.zip',$tmpServerDir)) return  $this->jsonError('下载升级文件失败!');
        //然后把端口写修改了
        $configFile=$tmpServerDir.'/easyswoole/produce.php';
        if (!is_file($configFile)) return  $this->jsonError('没有找到配置文件');
        $baseConfig=file_get_contents($configFile);
        //替换端口
        if (in_array(System::config(K::KEY_SERVER_ID),K::CONST_INNER_SERVER_ID)){
            $baseConfig=str_replace(['9402','9502','250000','swoole_cpu_num()*2'],[$data[K::KEY_SERVER_PORT],$data[K::KEY_SERVER_PORT],'250000',1],$baseConfig);
        }else{
            $baseConfig=str_replace(['9402','9502','250000','swoole_cpu_num()*2'],[$data[K::KEY_SERVER_PORT],$data[K::KEY_SERVER_PORT],'150000',1],$baseConfig);
        }
        if (!$baseConfig) return $this->jsonError('配置文件替换失败!');
        file_put_contents($configFile,$baseConfig);
        if (!Shell::shellKill('bigemap'.$data[K::KEY_SERVER_PORT])) return  $this->jsonError('停止旧程序失败');
        Coroutine::sleep(1);
        if (!Shell::shellStart('mv '.$serverDir.' '.$tmpCacheDir.'/'.$data[K::KEY_SERVER_PORT].'_'.date('Y-m-d-H-i-s'))) return $this->jsonError('移动旧的文件夹失败');
        Coroutine::sleep(0.3);
        if (!Shell::shellStart('mv '.$tmpServerDir.'/easyswoole '.$serverDir)) return $this->jsonError('移动升级文件失败!');
        Coroutine::sleep(0.2);
        if (!Shell::shellStart('php '.$serverDir.'/easyswoole server start -d -mode=produce')) return  $this->jsonError('开启程序失败');
        return  $this->jsonSuccess('更新成功!');
    }

    public function proxyCheck(){
        //这里的这个方法主要是在docker 在刚刚启动的时候，所有的代理都没有启用，在这个时候服务器的压力会很大。同时，内网中也很容易代理的复用，导致无效的结果
        $getData=$this->getRequestData();
        if (!$getData['port']){
            return $this->jsonError('no port');
        }
        if ($getData['port']=='9502') return $this->jsonSuccess('go');
        $table=TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG);
        $runningData=$table->get(K::KEY_SUB_SERVER_V2RAY_RUNNING,'data');
        $runningData=json_decode($runningData,true);
        if (!$runningData) $runningData=[];
        $serverId=System::config(K::KEY_SERVER_ID);
        $subServer=\App\Tools\Proxy::subServer();
        $ports=array_column($subServer,'port');
        if (in_array($serverId,K::CONST_INNER_SERVER_ID)){
            //这里是内网中的主服务器,同时允许7个子服务器检查代理
            $limit=2;
        }else{
            //只允许2个代理检测
            $limit=2;
        }
        if (!$runningData[$getData['port']]){
            $runningData[$getData['port']]=['time'=>time(),'time_str'=>date('Y-m-d H:i:s')];
        }
        $runningData=array_filter($runningData,function ($v){
            //如果有代理检查时间已经超过了半小时，那么将这个服务器移出使用
           return time()-$v['time']<60*30;
        });
        //然后和本地已经有的子服务器来比较，求出还没有提交的服务器
        $runningData=array_diff_key($runningData,array_flip($ports));
        if (count($runningData)>$limit){
            return $this->jsonError($runningData);
        }
        $table->set(K::KEY_SUB_SERVER_V2RAY_RUNNING,['data'=>json_encode($runningData),'number'=>1]);
        return  $this->jsonSuccess('go');
    }
}
