<?php

namespace App\Tools;

use App\Key\K;
use EasySwoole\Component\Timer;
use EasySwoole\EasySwoole\ServerManager;
use SQLite3;
use Swlib\Saber;
use Swlib\SaberGM;
use Swoole\Coroutine;

class StreetViewDownloader
{
//    private $tileUrl = 'https://streetviewpixels-pa.googleapis.com/v1/tile?cb_client=apiv3&panoid={id}&output=tile&x={x}&y={y}&zoom={z}&nbt=1&fover=2';
    private $tileUrl = '/v1/tile?cb_client=apiv3&panoid={id}&output=tile&x={x}&y={y}&zoom={z}&nbt=1&fover=2';
    /**
     * @var SQLite3
     */
    private $sqlite;

    private $sqliteConfig = [
        'ok' => false,
        'name' => '',
        'sum' => 0,
        'dir' => '',
        'limit' => 4000 * 3.5,
    ];

    private $wait = [
        'timeout' => 5 * 60,
        'prev_time' => 0
    ];

    private $workerId;
    private $speed = [
        'sum' => 0,//
        'total'=>0,//
        'ok'=>0,
        'has'=>0,
        'commit'=>0,
        'minute' => '',
        'ids' => '',
    ];

    private $thread = [
        'sum' => 4,
        'done'=>0,
        'index' => 0
    ];
    private $work = [
        'url' => []
    ];

    private $error = [
        'sum' => 0,
        'list' => [],
        'limit' => 40000
    ];
    /**
     * @var Saber
     */
    private $saber;
    private $zoomList = [
        '0' => ['x' => 0, 'y' => 0, 'need' => true],
        '1' => ['x' => 1, 'y' => 0, 'need' => true],
        '2' => ['x' => 3, 'y' => 1, 'need' => true],
        '3' => ['x' => 6, 'y' => 3, 'need' => true],
        '4' => ['x' => 12, 'y' => 6, 'need' => true],
        '5' => ['x' => 25, 'y' => 12, 'need' => true],
    ];

    public function __construct()
    {
        $this->sqliteConfig['dir'] = dirname(EASYSWOOLE_ROOT) . '/db';
        if (!is_dir($this->sqliteConfig['dir'])) mkdir($this->sqliteConfig['dir']);
    }

    public function initSqlite()
    {
        $dbDir = $this->sqliteConfig['dir'];
        $this->sqliteConfig['name'] = date('Y-m-d-H-i-s') . '=' . strtolower(uniqid()) . '.db';
        $this->sqliteConfig['file_dir'] = $dbDir . '/' . $this->sqliteConfig['name'];
        $this->sqlite = new SQLite3($dbDir . '/' . $this->sqliteConfig['name']);
        $this->sqlite->query('CREATE TABLE "tile" (
            "id"	TEXT UNIQUE,
            "data"	BLOB,
            PRIMARY KEY("id")
        )');
        if ($this->sqlite->lastErrorCode()) {
            System::log('create table error', $this->sqlite->lastErrorMsg());
            $this->sqliteConfig['ok'] = false;
            return;
        }
        $this->sqliteConfig['ok'] = true;
//        $this->sqlite->query('PRAGMA temp_store = 2');
    }

    public function workerStart($workerId)
    {
        $this->workerId = $workerId;
        //先开启对应的协程，开始下载
        if ($workerId === 0) {
            echo 'start worker _id',PHP_EOL;
            $this->saber=Saber::create([
                'base_uri' => 'https://streetviewpixels-pa.googleapis.com',
                'use_pool' => 200,
            ]);
            $this->initSqlite();
            $this->sqlite->query('begin');
            //先开启几个任务来下载数据
            $this->subWorker();
            Timer::getInstance()->loop(2000, [$this, 'noticeError']);
            while (true) {
                $this->getWork();
                Coroutine::sleep(5);
            }
        }
    }

    public function saveDbData($id, $png)
    {
        $sql = 'select id from tile where id=\'' . $id . '\'';
        $has = $this->sqlite->query($sql);
        if ($this->sqlite->lastErrorCode()) {
            echo $this->sqlite->lastErrorMsg(), PHP_EOL;
            $this->error['sum']++;
            $this->error['list'][] = ['id' => $id, 'from' => 'insert-check'];
            return;
        }
        $has = $has->fetchArray(SQLITE3_ASSOC);
        if ($has['id']) {
            $this->speed['has']++;
            return;
        }

        $st = $this->sqlite->prepare('insert into tile (id,data) values (?,?)');
        $st->bindValue('1', $id, SQLITE3_TEXT);
        $st->bindValue('2', $png, SQLITE3_BLOB);
        $st->execute();
        if ($this->sqlite->lastErrorCode()) {
            $this->error['sum']++;
            $this->error['list'][] = ['id' => $id, 'from' => 'insert'];
            echo $this->sqlite->lastErrorMsg(), PHP_EOL;
        }
        $this->speed['ok']++;
    }

    public function getWork()
    {
        $freeSpace = disk_free_space('/');
        System::log('free space :' . $freeSpace / 1024 / 1024 / 1024);
        if ($freeSpace < 1024 * 1024 * 1024 * 5) {
            System::log('too disk space');
            return;
        }
//        echo 'count ',count($this->work['url']).' total '.$this->speed['total'],PHP_EOL;
        if (count($this->work['url']) === 0) {
            //当前已经下载完成，那么需要进行将下载的数据进行提交
            if (!$this->noticeDownloadDbs()){
                return;
            }
            if (!$this->noticeDownloadIds()) {
                return;
            }
            $speed=$this->speed['sum']/(microtime(true)-$this->speed['time']);
            $log=['speed=>'.$speed,'ok=>'.$this->speed['ok'],'has=>'.$this->speed['has'],'sum=>'.$this->speed['sum'],'total=>'.$this->speed['total']];
            $log=implode(' , ',$log);
            System::log($log);
            if ($this->speed['total']>$this->sqliteConfig['limit']){
                $this->speed['total']=0;
                Coroutine::sleep(0.5);
                $this->sqlite->query('commit');
                $this->sqlite->close();
                $this->speed['commit']++;
                if ($this->speed['commit']>=3){
                    Coroutine::sleep(1);
                    ServerManager::getInstance()->getSwooleServer()->stop();
                    return;
                }
                $this->initSqlite();
                System::log('new db');
                $this->sqlite->query('begin');
            }

            $this->speed['ok'] = 0;
            $this->speed['has'] = 0;
            $this->speed['sum'] = 0;
            $this->speed['time'] = microtime(true);
            $this->createTableData();
        }
    }

    public function createTableData()
    {
        try {
            $data = SaberGM::get(K::MANAGE_SERVER_ADDR_9502 . '/street/work/index', ['timeout' => 5]);
            $all = $data->getBody()->getContents();
            $all = json_decode($all, true);
            //将当前的ID要缓存起来，这样在下载完成 之后，要通知主服务器
            $this->speed['ids'] = $all['data']['ids'];
            foreach ($all['data']['ids'] as $v) {
                $tiles = $this->createTiles($v['id']);
                $this->speed['sum'] += count($tiles);
                foreach ($tiles as $vv) {
                    $this->work['url'][] = $vv;
                }
            }
            echo 'current  loop sum,', $this->speed['sum'], PHP_EOL;
        } catch (\Throwable $throwable) {
            System::log('get curl url failed ', $throwable->getMessage());
        }
    }

    public function createTiles($id)
    {
        $r = [];
        foreach ($this->zoomList as $z => $v) {
            if ($v['need']) {
                for ($i = 0; $i <= $v['x']; $i++) {
                    for ($j = 0; $j <= $v['y']; $j++) {
                        $r[] = [
                            'id' => $id,
                            'tile_id' => implode('-', [$id, $z, $i, $j]),
                            'url' => str_replace(['{id}', '{z}', '{x}', '{y}'], [$id, $z, $i, $j], $this->tileUrl),
                        ];
                    }
                }
            }
        }
        return $r;
    }

    public function curl($data)
    {
        try {
            $result=$this->saber->get($data['url'],['timeout'=>10]);
//            $result = SaberGM::get($data['url'], ['timeout' => 20]);
            return $result->getBody()->getContents();
        } catch (\Throwable $throwable) {
            $code = $throwable->getCode();
            echo 'http error code : ', $code, '   ', $throwable->getMessage(), PHP_EOL;
            switch ($code) {
                case -1:
                case -2:
                case -4:
                case -3:
                    $this->work['url'][] = $data;
                    break;
                case 400:
                    //这个是一个特殊的错误，此时需要把这个数据给提交主服务器
                    $this->error['list'][] = ['id' => $data['tile_id'], 'data' => $data, 'from' => '400'];
                    foreach ($this->work['url'] as $k => $v) {
                        if (strpos($v['url'], $data['id']) !== false) unset($this->work['url'][$k]);
                    }
                    break;
                default:
                    $this->error['list'][] = ['id' => $data['tile_id'], 'data' => $data, 'from' => 'download', 'code' => $code];
                    echo 'worker_id ', $this->workerId, ' === ', $throwable->getMessage(), PHP_EOL;
                    $this->error['sum']++;
            }
        }
        return false;
    }

    public function subWorker()
    {
        for ($i = 0; $i < $this->thread['sum']; $i++) {
            go([$this, 'download']);
        }
    }

    public function download()
    {
        $size = count($this->work['url']);
        if ($size && $size % 400 === 0) {
            echo 'count ', $size, ' error', $this->error['sum'], PHP_EOL;
        }
        if (!$size) {
            Coroutine::sleep(2);
            unset($size);
            $this->download();
        } else {
            if ($this->error['sum'] > $this->error['limit']) {
                echo 'too many errors', PHP_EOL;
                return;
            }

            if (!$this->sqliteConfig['ok']) {
                System::log('sqlite is error wait');
                Coroutine::sleep(5);
                $this->download();
            } else {
                $data = array_shift($this->work['url']);
                if ($data) {
                    $png=$this->curl($data);
                    if ($png) {
                        //注意，这个total 仅仅是表示总数
                        $this->speed['total']++;
                        $this->saveDbData($data['tile_id'],$png);
                        if ($this->speed['total'] && $this->speed['total'] % 200 === 0) {
//                            echo 'commit',PHP_EOL;
                            $this->sqlite->query('commit');
                            $this->sqlite->query('begin');
                        }
                    }
                    unset($png);
                }
                unset($data);
                $this->download();
            }
        }
        unset($size);
    }

    private function noticeDownloadIds()
    {
        $result = false;
        try {
            SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/street/work/confirm', ['ids' => $this->speed['ids']], ['timeout' => 5]);
            $result = true;
        } catch (\Throwable $throwable) {
            System::log('upload download ids error', $throwable->getMessage());
        }
        return $result;
    }

    public function noticeError()
    {
        if (!$this->error['list']) return;
        $post = $this->error['list'];
        try {
            //因为使用了定时器，防止在提交的过程，将新产生的数据丢弃
            $this->error['list'] = [];
            SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/street/work/error', $post, ['timeout' => 5]);
        } catch (\Throwable $throwable) {
            $this->error['list'] = array_merge($post, $this->error['list']);
            System::log('upload error list error ' . $throwable->getMessage());
        }
    }

    private function noticeDownloadDbs()
    {
        $result = false;
        try {
            $dbs = scandir($this->sqliteConfig['dir']);
            $dbs = array_filter($dbs, function ($v) {
                return strtolower(pathinfo($v, PATHINFO_EXTENSION)) === 'db';
            });
            foreach ($dbs as $k => $v) {
                if ($v === $this->sqliteConfig['name']) unset($dbs[$k]);
            }
            $dbs = array_values($dbs);
            $response = SaberGM::post(K::MANAGE_SERVER_ADDR_9502 . '/street/work/db', ['dbs' => $dbs], ['timeout' => 5]);
            $str = $response->getBody()->getContents();
            $str = json_decode($str, true);
            foreach ($str['data']['del'] as $d) {
                //要删除的数据
                $un = unlink($this->sqliteConfig['dir'] . '/' . $d) ? 'success' : 'failed';
                System::log('unlink ' . $d . ' status: ' . $un);
            }
            $result = true;
        } catch (\Throwable $throwable) {
            System::log('upload download dbs error', $throwable->getMessage());
        }
        return $result;
    }
}
