<?php

namespace App\Tools;

use App\Key\K;
use App\WebSocket\WebSocket;
use EasySwoole\Component\TableManager;
use EasySwoole\Component\Timer;
use EasySwoole\EasySwoole\Config;
use EasySwoole\EasySwoole\ServerManager;
use Swlib\Http\Exception\BadResponseException;
use Swlib\Saber;
use Swlib\SaberGM;
use Swoole\Coroutine;

class System
{
    /**
     * @param bool $userPool
     * @return Saber
     */
    public static function getSaber($userPool=true)
    {
        $config = static::config();
        if ($config[K::KEY_PROXY_HISTORY_ENABLE] && $config[K::KEY_PROXY_HISTORY]) {
            $saber = Saber::create([
                'base_uri' => 'https://kh.google.com',
                'use_pool' => $userPool,
                'proxy' => $config[K::KEY_PROXY_HISTORY],
            ]);
        } else {
            $saber = Saber::create([
                'base_uri' => 'https://kh.google.com',
                'use_pool' => $userPool,
            ]);
        }
        return  $saber;
    }

    public static function incr($key, $number = 1)
    {
        $system = TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG);
        return $system->incr($key, 'number', $number);
    }
    public static function log($data, $msg = '', $is_dump = false)
    {
        $s = explode('.', round(microtime(true), 3));
        echo PHP_EOL, date('Y-m-d H:i:s'), ' ', end($s), '  ', $msg, PHP_EOL;
        $is_dump ? var_dump($data) : print_r($data);
        echo PHP_EOL;
    }
    public static function getServerIdNumber($serverId='')
    {
        if (!$serverId) $serverId = System::config(K::KEY_SERVER_ID);
        return intval(str_replace(['history'], [''], $serverId));
//        $serverId = ;
//        $serverId = str_pad($serverId, 2, '0', STR_PAD_LEFT);
//        return  $serverId;
//        return substr(md5(uniqid()), 0, 2) . $serverId . substr(md5(uniqid()), 4, 10);
    }

    public static function getServerInfo()
    {
        $configTable = TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG);
        $postData = System::config();
        $isNewDay = false;
        foreach ([K::KEY_EVERY_DAY_UPLOAD, K::KEY_EVERY_DAY_POST1, K::KEY_EVERY_DAY_POST2, K::KEY_EVERY_SECOND_REQUEST_ALL_CACHE, K::KEY_EVERY_SECOND_REQUEST_OK_CACHE, K::KEY_GOOGLE_HISTORY_FORBIDDEN_TIMES] as $v) {
            $data = $configTable->get($v);
            if (in_array($v, [K::KEY_EVERY_DAY_UPLOAD, K::KEY_EVERY_DAY_POST1, K::KEY_EVERY_DAY_POST2])) {
                if ($data['data'] != date('Y-m-d')) {
                    //这些数据是每天要清空的
                    if (!$isNewDay) $isNewDay = $data['data'];
                    $configTable->set($v, ['data' => date('Y-m-d'), 'number' => 0]);
                }
            }
            file_put_contents($postData[K::KEY_CACHE_PATH] . '/' . $v, json_encode($data));
            $postData[$v] = $data;
        }
        try {
            if (time()%1800==0){
                $managerVersion=SaberGM::get('http://127.0.0.1:9504?action_name=version&bigemap=1',['timeout'=>1]);
                $postData['manager_version']=$managerVersion->getBody()->getContents();
                if ($postData[K::KEY_PROXY_HISTORY_ENABLE]){
                    $ip=Proxy::getProxyIp($postData[K::KEY_PROXY_HISTORY]);
                    if ($ip) System::config(K::KEY_PROXY_IP,$ip);
                }
            }
        }catch (\Throwable $exception){

        }
        $subServer=Proxy::subServer();
        $postData['sub_server']=count($subServer);
        $postData['waiting_length']=TableManager::getInstance()->get(K::TABLE_QUERY_WAITING)->count();
        $postData['second_length'] = TableManager::getInstance()->get(K::TABLE_SECOND_CACHE)->count();
        $postData['connections'] = count(ServerManager::getInstance()->getSwooleServer()->connections);
        $postData['time'] = time();
        if ($isNewDay) {
            //这里时间要使用上一天的时间
            file_put_contents($postData[K::KEY_CACHE_PATH] . '/day', json_encode(array_merge($postData, ['time' => $isNewDay]), JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES) . PHP_EOL, FILE_APPEND);
        }
        return $postData;
    }

    public static function serverData($ip = '', $data = '')
    {
        $path = static::config(K::KEY_CACHE_PATH) . '/ip';
        if (!is_dir($path)) mkdir($path);
        $ips = scandir($path);
        $all = [];
        foreach ($ips as $v) {
            if (strtolower(pathinfo($v, PATHINFO_EXTENSION)) === 'ip') {
                $current = file_get_contents($path . '/' . $v);
                $current = json_decode($current, true);
                if ($current) {
                    $k = pathinfo($v, PATHINFO_FILENAME);
                    $all[$k] = $current;
                }
            }
        }
        if (!$ip) return $all;
        if ($data) {
            $old = $all[$ip];
            //然后要保证已经保存的数据
            if ($old['time']>$data['time']) {
                return  $old;
            }
            if (is_array($old)) $data = array_merge($old, $data);
            file_put_contents($path . '/' . $ip . '.ip', json_encode($data, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES));
            return $data;
        }
        return $all[$ip];
    }
    public static function config($key = '', $value = null)
    {
        $table = TableManager::getInstance()->get(K::TABLE_SYSTEM_CONFIG);
        $config = $table->get(K::KEY_SYSTEM_CONFIG);
        $config = json_decode($config['data'], true);
        if (!$config) {
            $systemConfig=Config::getInstance()->getConf('MAIN_SERVER');
            $config = [
                K::KEY_CACHE_PATH => dirname(EASYSWOOLE_ROOT) . '/cache/'.$systemConfig['PORT'],
                K::KEY_EVERY_SECOND_LIMIT => 30,
                K::KEY_SERVER_ID => '',
                K::KEY_SERVER_PORT=>$systemConfig['PORT'],
                K::KEY_PROXY_HISTORY => '',
                K::KEY_PROXY_HISTORY_ENABLE => '',
                K::KEY_FIXED_SECOND_QUERY => 1,
                K::KEY_SERVER_BUYER => '',
                K::KEY_SERVER_DEADLINE => 0,
                K::KEY_GOOGLE_HISTORY_ENABLE => 1,
                K::KEY_HISTORY_MOBILE_ENABLE => 1,
                K::KEY_FIRST_POST_PASS=>0,
                K::KEY_REQUEST_WAITING_ENABLE=>'',
            ];
            //只有在系统启动的时候才会读取配置文件
            if (!is_dir($config[K::KEY_CACHE_PATH])&&!mkdir($config[K::KEY_CACHE_PATH],0777,true)) System::log('mkdir cache dir failed');
            $fileConfig = file_get_contents($config[K::KEY_CACHE_PATH] . '/config');
            $fileConfig = json_decode($fileConfig, true);
            if ($fileConfig) {
                //由于升级的时候，缓存的文件夹修改过。然后要保证缓存的文件夹正常
                if ($config[K::KEY_CACHE_PATH]!=$fileConfig[K::KEY_CACHE_PATH]) $fileConfig[K::KEY_CACHE_PATH]=$config[K::KEY_CACHE_PATH];
                $config = array_merge($config, $fileConfig);
            }
            $config[K::KEY_LAST_REBOOT_TIME] = time();
            $bigemapData = Config::getInstance()->getConf('BIGEMAP_DATA');
            $config[K::KEY_SYSTEM_VERSION] = $bigemapData['version'];
            //然后再保存一次数据
            $table->set(K::KEY_SYSTEM_CONFIG, ['data' => json_encode($config, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES), 'number' => time()]);
        }

        if (!$key) return $config;
        if (!is_null($value)) {
//            $reflect=new \ReflectionClass(K::class);
//            //只能设置已经定义好的方法
//            if (!in_array($key,array_values($reflect->getConstants()))) return false;
            $config[$key] = $value;
            switch ($key) {
                case K::KEY_ENABLE_STREET_VIEW_DOWNLOAD:
                    //如果是启用了街景，那么服务器会重启一次
                    if ($config[$key]){
                        Timer::getInstance()->after(1000,function (){
                            ServerManager::getInstance()->getSwooleServer()->stop();
                        });
                    }
                    break;
                case K::KEY_GOOGLE_HISTORY_ENABLE:
                    if ($config[$key]) {
                        //如果是启用了地图，那么要删除可能出现的错误次数
                        $table->del(K::KEY_GOOGLE_HISTORY_FORBIDDEN_TIMES);
                        $table->del(K::KEY_TILE_ERROR_TIMES);
                        //$config[K::KEY_GOOGLE_HISTORY_DISABLED_TIME] = '';
                    } else {
                        //如果暂停了，那么需要保存一次暂停的时间
                        $config[K::KEY_GOOGLE_HISTORY_DISABLED_TIME] = time();
                        if ($config[K::KEY_PARENT_IP]){
                            //如果有父级IP，那么要通知父级IP,删除当前这个子服务器
                            try {
                                SaberGM::delete($config[K::KEY_PARENT_IP].'/google/api/sub?bigemap=1&port='.$config[K::KEY_SERVER_PORT],['timeout'=>1]);
                            }catch (\Throwable $throwable){
                                static::log($throwable->getMessage(),'notice parent server failed');
                            }
                        }
                        //清空代理的IP
//                        $config[K::KEY_PROXY_IP]='';
                    }
                    break;
                case K::KEY_REQUEST_WAITING_ENABLE:
                    if (!$config[$key]){
                        //这里表示不再使用缓冲区，那么直接需要所有的缓冲区的数据都设置成500；
                        $waitingTable=TableManager::getInstance()->get(K::TABLE_QUERY_WAITING);
                        $del=[];
                        $server=ServerManager::getInstance()->getSwooleServer();
                        $workerId=$server->worker_id;
                        foreach ($waitingTable as $k=>$v){
                            $del[]=$k;
                            if ($workerId==$v['worker_id']){
                                Coroutine::resume($v['cid']);
                            }else{
                                $server->sendMessage(['action'=>'allow','cid'=>$v['cid']],$v['worker_id']);
                            }
                        }
                        foreach ($del as $v){
                            $waitingTable->del($v);
                        }
                    }
            }
            $table->set(K::KEY_SYSTEM_CONFIG, ['data' => json_encode($config, JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES), 'number' => time()]);
            //同时保存一次配置信息
            file_put_contents($config[K::KEY_CACHE_PATH] . '/config', json_encode($config, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE));
        }
        return $config[$key];
    }
}
